/*************************************************
* Public Key Base Class Header File              *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PKBASE_H__
#define OPENCL_PKBASE_H__

#include <opencl/config.h>
#include <opencl/secmem.h>

namespace OpenCL {

/*************************************************
* Public Key Cryptosystem Base Class             *
*************************************************/
struct PK_Cryptosystem
   {
   virtual u32bit max_input() const = 0;
   virtual ~PK_Cryptosystem() {}
   };

/*************************************************
* Public Key Encryptor Base Class                *
*************************************************/
struct PK_Encryptor : public PK_Cryptosystem
   {
   virtual SecureVector<byte> encrypt(const byte[], u32bit) const = 0;
   virtual ~PK_Encryptor() {}
   };

/*************************************************
* Public Key Verifier Base Class                 *
*************************************************/
struct PK_Verifier : public PK_Cryptosystem
   {
   virtual bool verify(const byte[], u32bit, const byte[], u32bit) const = 0;
   virtual ~PK_Verifier() {}
   };

/*************************************************
* Public Key Decryptor Base Class                *
*************************************************/
struct PK_Decryptor : public PK_Encryptor
   {
   virtual SecureVector<byte> decrypt(const byte[], u32bit) const = 0;
   virtual ~PK_Decryptor() {}
   };

/*************************************************
* Public Key Signer Base Class                   *
*************************************************/
struct PK_Signer : public PK_Verifier
   {
   virtual SecureVector<byte> sign(const byte[], u32bit) const = 0;
   virtual ~PK_Signer() {}
   };

}

#endif
