/*************************************************
* Disk Storage Allocator Header File             *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_MMAP_ALLOCATION_H__
#define OPENCL_MMAP_ALLOCATION_H__

#include <opencl/manalloc.h>
#include <opencl/mutex.h>
#include <map>
#include <vector>
#include <utility>

namespace OpenCL {

/*************************************************
* Mmap Disk Allocator                            *
*************************************************/
class MmapDisk_Allocator : public ManagedAllocator
   {
   public:
      void* alloc_block(u32bit) const;
      void dealloc_block(void*, u32bit) const;
      SecureAllocator* clone() { return this; }
      MmapDisk_Allocator(const std::string& = "");
      ~MmapDisk_Allocator();
   private:
      static const u32bit CACHE_SIZE = 2;
      struct Maped_Buffer { void* buf; u32bit length; int fd; };

      u32bit getpagesize() const;
      void* mmap_block(u32bit) const;
      void munmap_block(void*, u32bit) const;

      std::string path;
      mutable std::map<void*, std::pair<int, u32bit> > allocated;
      mutable std::vector<Maped_Buffer> cache;
      mutable Mutex* lock;
   };

}

#endif
